@@ -3,6 +3,7 @@
 
 """
 
+import re
 import copy
 import binascii
 import string
@@ -10,13 +11,16 @@ import sys
 import argparse
 from collections import namedtuple
 from io import StringIO
+from io import BytesIO
 
 from humanfriendly import format_size
 from argparse_addons import Integer
+from elftools.elf.elffile import ELFFile
+from elftools.elf.constants import SH_FLAGS
 
 
 __author__ = 'Erik Moqvist'
-__version__ = '17.9.0'
+__version__ = '17.11.0'
 
 
 DEFAULT_WORD_SIZE_BITS = 8
@@ -290,6 +294,22 @@ def pretty_ti_txt(line):
     return line
 
 
+def comment_remover(text):
+    def replacer(match):
+        s = match.group(0)
+
+        if s.startswith('/'):
+            return " " # note: a space and not an empty string
+        else:
+            return s
+
+    pattern = re.compile(
+        r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"',
+        re.DOTALL | re.MULTILINE)
+
+    return re.sub(pattern, replacer, text)
+
+
 def is_srec(records):
     try:
         unpack_srec(records.partition('\n')[0].rstrip())
@@ -414,8 +434,8 @@ class _Segment:
         """
 
         if ((minimum_address >= self.maximum_address)
-            and (maximum_address <= self.minimum_address)):
-            raise Error('cannot remove data that is not part of the segment')
+            or (maximum_address <= self.minimum_address)):
+            return
 
         if minimum_address < self.minimum_address:
             minimum_address = self.minimum_address
@@ -584,19 +604,13 @@ class _Segments:
         new_list = []
 
         for segment in self._list:
-            if (segment.maximum_address <= minimum_address
-                or maximum_address < segment.minimum_address):
-                # No overlap.
+            split = segment.remove_data(minimum_address, maximum_address)
+
+            if segment.minimum_address < segment.maximum_address:
                 new_list.append(segment)
-            else:
-                # Overlapping, remove overwritten parts segments.
-                split = segment.remove_data(minimum_address, maximum_address)
 
-                if segment.minimum_address < segment.maximum_address:
-                    new_list.append(segment)
-
-                if split:
-                    new_list.append(split)
+            if split:
+                new_list.append(split)
 
         self._list = new_list
 
@@ -974,6 +988,46 @@ class BinFile:
         if not eof_found:
             raise Error("missing file terminator")
 
+    def add_verilog_vmem(self, data, overwrite=False):
+        address = None
+        chunk = b''
+        words = re.split(r'\s+', comment_remover(data).strip())
+        word_size_bytes = None
+
+        for word in words:
+            if not word.startswith('@'):
+                length = len(word)
+
+                if (length % 2) != 0:
+                    raise Error('Invalid word length.')
+
+                length //= 2
+
+                if word_size_bytes is None:
+                    word_size_bytes = length
+                elif length != word_size_bytes:
+                    raise Error(
+                        f'Mixed word lengths {length} and {word_size_bytes}.')
+
+        for word in words:
+            if word.startswith('@'):
+                if address is not None:
+                    self._segments.add(_Segment(address,
+                                                address + len(chunk),
+                                                chunk,
+                                                self.word_size_bytes))
+
+                address = int(word[1:], 16) * word_size_bytes
+                chunk = b''
+            else:
+                chunk += bytes.fromhex(word)
+
+        if address is not None and chunk:
+            self._segments.add(_Segment(address,
+                                        address + len(chunk),
+                                        chunk,
+                                        self.word_size_bytes))
+
     def add_binary(self, data, address=0, overwrite=False):
         """Add given data at given address. Set `overwrite` to ``True`` to
         allow already added data to be overwritten.
@@ -987,15 +1041,46 @@ class BinFile:
                                     self.word_size_bytes),
                            overwrite)
 
+    def add_elf(self, data, overwrite=True):
+        """Add given ELF data.
+
+        """
+
+        elffile = ELFFile(BytesIO(data))
+
+        self.execution_start_address = elffile.header['e_entry']
+
+        for segment in elffile.iter_segments():
+            if segment['p_type'] != 'PT_LOAD':
+                continue
+
+            address = segment['p_paddr']
+            offset = segment['p_offset']
+            size = segment['p_filesz']
+
+            if size == 0:
+                continue
+
+            self._segments.add(_Segment(address,
+                                        address + size,
+                                        data[offset:offset + size],
+                                        self.word_size_bytes),
+                               overwrite)
+
     def add_file(self, filename, overwrite=False):
         """Open given file and add its data by guessing its format. The format
-        must be Motorola S-Records, Intel HEX or TI-TXT. Set `overwrite` to
-        ``True`` to allow already added data to be overwritten.
+        must be Motorola S-Records, Intel HEX, TI-TXT. Set `overwrite`
+        to ``True`` to allow already added data to be overwritten.
 
         """
 
         with open(filename, 'r') as fin:
-            self.add(fin.read(), overwrite)
+            try:
+                data = fin.read()
+            except UnicodeDecodeError:
+                raise UnsupportedFileFormatError()
+
+        self.add(data, overwrite)
 
     def add_srec_file(self, filename, overwrite=False):
         """Open given Motorola S-Records file and add its records. Set
@@ -1025,6 +1110,15 @@ class BinFile:
         with open(filename, 'r') as fin:
             self.add_ti_txt(fin.read(), overwrite)
 
+    def add_verilog_vmem_file(self, filename, overwrite=False):
+        """Open given Verilog VMEM file and add its contents. Set `overwrite` to
+        ``True`` to allow already added data to be overwritten.
+
+        """
+
+        with open(filename, 'r') as fin:
+            self.add_verilog_vmem(fin.read(), overwrite)
+
     def add_binary_file(self, filename, address=0, overwrite=False):
         """Open given binary file and add its contents. Set `overwrite` to
         ``True`` to allow already added data to be overwritten.
@@ -1034,6 +1128,15 @@ class BinFile:
         with open(filename, 'rb') as fin:
             self.add_binary(fin.read(), address, overwrite)
 
+    def add_elf_file(self, filename, overwrite=False):
+        """Open given ELF file and add its contents. Set `overwrite` to
+        ``True`` to allow already added data to be overwritten.
+
+        """
+
+        with open(filename, 'rb') as fin:
+            self.add_elf(fin.read(), overwrite)
+
     def as_srec(self, number_of_data_bytes=32, address_length_bits=32):
         """Format the binary file as Motorola S-Records records and return
         them as a string.
@@ -1229,6 +1332,25 @@ class BinFile:
 
         return '\n'.join(lines) + '\n'
 
+    def as_verilog_vmem(self):
+        """Format the binary file as a Verilog VMEM file and return it as a string.
+
+        >>> print(binfile.as_verilog_vmem())
+
+        """
+
+        lines = []
+
+        if self._header is not None:
+            lines.append(f'/* {self.header} */')
+
+        for segment in self._segments:
+            for address, data in segment.chunks(24):
+                data_hex = ' '.join(f'{byte:02X}' for byte in data)
+                lines.append(f'@{address:08X} {data_hex}')
+
+        return '\n'.join(lines) + '\n'
+
     def as_binary(self,
                   minimum_address=None,
                   maximum_address=None,
@@ -1617,7 +1739,7 @@ def _convert_input_format_type(value):
                     f"invalid binary address '{items[1]}'")
 
         args = (address, )
-    elif fmt in ['ihex', 'srec', 'auto', 'ti_txt']:
+    elif fmt in ['ihex', 'srec', 'auto', 'ti_txt', 'verilog_vmem', 'elf']:
         pass
     else:
         raise argparse.ArgumentTypeError(f"invalid input format '{fmt}'")
@@ -1649,6 +1771,8 @@ def _convert_output_format_type(value):
                     f"invalid {fmt} address length of '{items[2]}' bits")
 
         args = (number_of_data_bytes, address_length_bits)
+    elif fmt == 'elf':
+        raise argparse.ArgumentTypeError(f"invalid output format '{fmt}'")
     elif fmt == 'binary':
         minimum_address = None
         maximum_address = None
@@ -1670,6 +1794,8 @@ def _convert_output_format_type(value):
         args = (minimum_address, maximum_address)
     elif fmt == 'hexdump':
         pass
+    elif fmt == 'verilog_vmem':
+        pass
     else:
         raise argparse.ArgumentTypeError(f"invalid output format '{fmt}'")
 
@@ -1684,7 +1810,10 @@ def _do_convert_add_file(bf, input_format, infile, overwrite):
             try:
                 bf.add_file(infile, *args, overwrite=overwrite)
             except UnsupportedFileFormatError:
-                bf.add_binary_file(infile, *args, overwrite=overwrite)
+                try:
+                    bf.add_elf_file(infile, *args, overwrite=overwrite)
+                except:
+                    bf.add_binary_file(infile, *args, overwrite=overwrite)
         elif fmt == 'srec':
             bf.add_srec_file(infile, *args, overwrite=overwrite)
         elif fmt == 'ihex':
@@ -1693,6 +1822,10 @@ def _do_convert_add_file(bf, input_format, infile, overwrite):
             bf.add_binary_file(infile, *args, overwrite=overwrite)
         elif fmt == 'ti_txt':
             bf.add_ti_txt_file(infile, *args, overwrite=overwrite)
+        elif fmt == 'verilog_vmem':
+            bf.add_verilog_vmem_file(infile, *args, overwrite=overwrite)
+        elif fmt == 'elf':
+            bf.add_elf_file(infile, *args, overwrite=overwrite)
     except AddDataError:
         sys.exit('overlapping segments detected, give --overwrite to overwrite '
                  'overlapping segments')
@@ -1711,6 +1844,8 @@ def _do_convert_as(bf, output_format):
         converted = bf.as_hexdump()
     elif fmt == 'ti_txt':
         converted = bf.as_ti_txt()
+    elif fmt == 'verilog_vmem':
+        converted = bf.as_verilog_vmem()
 
     return converted
 
@@ -1780,6 +1915,7 @@ def _do_as_hexdump(args):
         bf.add_file(binfile)
         print(bf.as_hexdump(), end='')
 
+
 def _do_as_ti_txt(args):
     for binfile in args.binfile:
         bf = BinFile()
@@ -1857,14 +1993,14 @@ def _main():
         action='append',
         default=[],
         type=_convert_input_format_type,
-        help=('Input format auto, srec, ihex, ti_txt, or binary[,<address>] '
-              '(default: auto). This argument may be repeated, selecting the '
-              'input format for each input file.'))
+        help=('Input format auto, srec, ihex, ti_txt, verilog_vmem, elf, or '
+              'binary[,<address>] (default: auto). This argument may be repeated, '
+              'selecting the input format for each input file.'))
     subparser.add_argument(
         '-o', '--output-format',
         default='hexdump',
         type=_convert_output_format_type,
-        help=('Output format srec, ihex, ti_txt, binary or hexdump '
+        help=('Output format srec, ihex, ti_txt, verilog_vmem, binary or hexdump '
               '(default: %(default)s).'))
     subparser.add_argument(
         '-s', '--word-size-bits',
